function [emX, emY, time] = EyePos (c, r)

% @cortex\eyepos.m
%
% returns EyePositions from a @cortex object, given a @key
%
% Syntax: [POSX, POSY, TIME] = EYEPOS (COBJ, KEY);
%
% where
% COBJ:  the @cortex object
% KEY:   one key from a @key object
%
% POSX:  horizontal projections of eye positions;
% POSY:  vertical   projections of eye positions;
%        each matrix has COL samples (in dva) by ROW trials;
%        values are time-locked on the cue(s) specified in the rule
% TIME:  the corresponding time scale
%
% Last modified: 28 Jan 04

% based on eogmatrix.m, last saved on 19 May 00
% 24 May 00: help text revised
% 28 Jan 04: @key support added

if isa (r, 'rules') | isa (r, 'key')
   yestrials = trialset (c, r);
else
   error ('2nd arg must be a rule');
end

sampling = unique (c.eog_rate(yestrials));
if length (sampling) > 1
   error ('variable sampling rate is not yet supported');
end

samples_per_row = fix ((r.finish-r.start)/sampling);
emX = zeros (length (yestrials), samples_per_row);
emY = zeros (length (yestrials), samples_per_row);
instance = 0;
included_trials = [];

for trial=yestrials
   % find the first and the last element of the vector 'e.eog_X'
   % that contains the samples for the current trial
   % remember that c.eog_start (trial) == 0 if
   % the current trial does not contain any eog values
   first = c.eog_ptr_start (trial);
   last  = c.eog_ptr_end   (trial);
   samples = last-first+1;
   % of all requested cues, find those that are actually present
   % in the current trial. THIS MAY BE SLOW!
   cues_found = intersect (r.cues, c.codes (trial, :));
   % warn user if no cue event is found
   if isempty (cues_found)
      disp (['no cue events found in trial ', num2str(trial)]);
   end
   % for each cue event found
   for j=cues_found
      % there should NOT be more than one instance of the same cue event!
      cue_index = find (c.codes (trial, :) == j);
      if length (cue_index)>1
         disp (['only the first instance of event ', num2str(j), ' will be considered']);
         cue_index = cue_index(1);
      end
      
      k = c.times (trial, cue_index);
      start_time = k + r.start;
      end_time   = k + r.finish;
      start_from_first = start_time - c.eog_start(trial);
      if (start_from_first < 0 | (end_time > (start_time + samples * sampling)))
         disp (['requested window exceeds boundaries of trial ', num2str(trial)]);
      else
         instance = instance + 1;
         included_trials = [included_trials; trial]; 
         findx = first + fix (start_from_first/sampling);
         lindx = findx + samples_per_row-1;
         emX (instance, :) = double(c.eog_X (findx:lindx))'/c.eog2dva;
         emY (instance, :) = double(c.eog_Y (findx:lindx))'/c.eog2dva;
      end
   end
end
emX = emX (1:instance, :);
emY = emY (1:instance, :);
t = r.start:sampling:r.finish;
time = t (1:length(t)-1);