function [spmat, included_trials] = getspikearray (c, cluster, k)

%@cortex/private/getspikearray.m
%
% returns a spike matrix for a single key.
%
% This is a private function, only available
% to @cortex methods
%
% Last modified: 3 Aug 02

% 18 Apr 02: tiny fixes while migrating to @keys
%  1 May 02: spike_raster indexing shifted by 1,
%            to allow spikes occurring at 0ms.
%  3 Aug 02: warning and help messages

if sum(k.size)>1
   error ('single key required');
end
included_trials = [];

% first retrieve the subset of trials to be included in the analysis
% this step greatly improves the execution speed, compared to the previous version
yestrials = trialset (c, k);
if isempty (yestrials)
   spmat = [];
   return;
end

% preallocate this array as if all trials were to be included
spmat = uint8 (zeros (length(yestrials), k.finish - k.start));
instance = 0;
cueless = [];
winless = [];

% find what position in the c.spikes array occupies the requested cluster
% also makes sure that cluster is actually present
cluster_pos = find (c.cluster_bank == cluster(1));
if isempty (cluster_pos)
   warning ('the requested cluster does not exist in this file');
   disp    ('returning a matrix with 0 spikes');
   return
end

% for each trial in the current cortex object
for trial=yestrials
   % find the first and the last element of the vector 'c.spikes'
   % that contains the spikes for the current trial and cluster
   % remember that c.spike_start (trial, cluster_pos) == 0 if
   % the current trial does not contain any cluster_pos spikes
   first = c.spike_start (trial, cluster_pos);
   last  = c.spike_end   (trial, cluster_pos);
   % initialize the vector that will eventually be appended to spmat
   last_time_stamp = max (c.times (trial, :));
   spike_raster = uint8 (zeros (1, last_time_stamp + 1));
   % do only if the current trial actually contains spikes
   % to avoid an error with: c.spikes (0:0)
   if first > 0
      spike_raster (double (c.spikes(first:last)) + 1) = 1;
   end
   % of all requested cues, find those that are actually present
   % in the current trial. THIS MAY BE SLOW!
   cues_found = intersect (k.cues, c.codes (trial, :));
   % warn user if no cue event is found
   if isempty (cues_found)
      cueless = [cueless trial];
   end
   % for each cue event found
   for j=cues_found
      % there may be more than one instance of the same cue event!
      cue_event_indices = find (c.codes (trial, :) == j);
      % for each occurrence of the same cue event...
      for w = c.times (trial, cue_event_indices)
         start_time = w + k.start;
         end_time   = w + k.finish-1;
         if (start_time < 1) | (end_time > last_time_stamp)
            winless = [winless trial];
         else
            instance = instance + 1;
            included_trials = [included_trials; trial];
            spmat (instance, :) = spike_raster (start_time+1:end_time+1);
         end
      end
   end
end
spmat = spmat (1:instance, :);

if ~isempty (cueless)
   warning ('could not retrieve all requested trials;');
   disp ('no cue events found in the following trial(s):');
   disp (cueless);
end
if ~isempty (winless)
   warning ('could not retrieve all requested trials;');
   disp ('time window exceeds boundaries of the following trial(s)');
   disp (winless);
end