function k = key (label, start, finish, cues, conditions, blocks, responses, trials, type, expect, rept)

% @key/key.m
% creates an object of class @key; this object is used to select subsets
% of trials and analysis time windows from a @cortex object.
%
% Syntax 1:
% KOBJ = KEY (LABEL, START, FINISH, CUES, CONDITIONS, BLOCKS, RESPONSES, TRIALS, TYPE, EXPECT, REPT)
% creates a one-key object and assigns the values passed as arguments
% to the corresponding fields.
%
% LABEL:         the name of the key;
%                defaults to 'noname' if an empty string is passed;
% START, FINISH: the offset in ms from the cue event;
% CUES:          an array of encodes to time-lock the analysis on;
% CONDITIONS:    see below
% BLOCKS:        see below
% RESPONSES:     see below
% TRIALS:        see below
% TYPE:          see below
% EXPECT:        see below
% REPT:          arrays of values to consider for the analysis; use -1 to include
%                all (e.g., if k.blocks == -1 then all blocks are included in the 
%                analysis performed by SPIKEMATRIX).
%
% Syntax 2:
% KOBJ = KEY
% creates a one-key object and sets its fields to the following default values:
% LABEL = 'noname', START = -50, FINISH = 250, CUES = 23, RESPONSES = 0;
% CONDITIONS = BLOCKS = TRIALS = TYPE = EXPECT = -1.
%
% Syntax 3:
% KOBJ = KEY (KEYFILE)
% creates a multiple-key object based on
% an ASCII file of name KEYFILE
%
% Once the object is created, all the fields
% listed above can be read/modified; for example:
%
%  kobj = key;
%  kobj.conditions = [4:8,12:16];
%  current_analysis = kobj.label;
%
% Use the property SIZE to get the number
% of keys and subkeys in an object:
%
%  ko = key ('groups.txt');
%  s = ko.size;
%  last_key = length (s);
%
% Last modified: 1 Dec 02

% 30 jan 99: last major changes
% 18 apr 01: added little check to avoid warnings: if ~isempty (lin), if lin == -1 ...
% 14 may 01: branch off to @key class
% 10 oct 02: help text fixed
%  1 Dec 02: help text fixed

switch nargin
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
case 0  % default object
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    k.size      =   1;
    k.label{1}  = 'noname';
    k.st.start  = -50;
    k.st.finish = 250;
    k.st.cues   =  23;
    k.st.conds  =  -1;
    k.st.blocks =  -1;
    k.st.resps  =   0;
    k.st.trials =  -1;
    k.st.type   =  -1;
    k.st.expect =  -1;
    k.st.rept   =  -1;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
case 11  % then use them all
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    k.size = 1;
    if isempty (label)
        k.label{1} = 'noname';
    else
        k.label{1} = label;
    end
    if isempty (start)
        k.st.start = -50;
    else
        k.st.start = start;
    end
    if isempty (finish)
        k.st.finish = 250;
    else
        k.st.finish = finish;
    end
    if isempty (cues)
        k.st.cues = 23;
    else
        k.st.cues = cues;
    end
    if isempty (conditions)
        k.st.conds = -1;   
    else
        k.st.conds = conditions;
    end
    if isempty (blocks)
        k.st.blocks = -1;
    else
        k.st.blocks = blocks;
    end
    if isempty (responses)
        k.st.resps = -1;
    else
        k.st.resps = responses;
    end
    if isempty (trials)
        k.st.trials = -1;
    else
        k.st.trials = trials;
    end
    if isempty (type)
        k.st.type = -1;
    else
        k.st.type = type;
    end
    if isempty (expect)
        k.st.expect = -1;
    else
        k.st.expect = expect;
    end
    if isempty (rept)
        k.st.rept = -1;
    else
        k.st.rept = rept;
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
case 1 % the argument is a file name
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    [list, rows, nfields, empty] = listfromfile (label);
    if empty, error ('all rows in key source file must contain the same number of fields'); end
    if ~strcmp (list {1,1}, 'label'), error ('first field in key file must be ''label'''); end
    if rows<2, error ('key source file must contain at least a header and one key'); end
    
    fields = list(1,:);
    keys   = list(2:end,:);
    keys_per_group = 0;
    group          = 0;
    old_label      ='';
    total_keys = rows - 1;
    
    for ck = 1:total_keys
        % if the current key label is different from the previous
        % then this is a new group (also applies to the very first key)
        if ~strcmp (keys{ck,1}, old_label)
            group = group + 1;
            k.size (group) = 0;
            k.label {group} = keys{ck,1};
        end
        k.size (group) = k.size (group) + 1;
        old_label = keys{ck,1};
        
        % every field in the current key is at first set to a default value
        k.st(ck).start  =   0;
        k.st(ck).finish = 300;
        k.st(ck).cues   =  23;
        k.st(ck).conds  =  -1;
        k.st(ck).blocks =  -1;
        k.st(ck).resps  =   0;
        k.st(ck).trials =  -1;
        k.st(ck).type   =  -1;
        k.st(ck).expect =  -1;
        k.st(ck).rept   =  -1;
        
        % loop through the fields actually present in the file
        % except for 'label', which is compulsory
        
        for fi=2:nfields
            v = keys{ck,fi};
            % try to assign the value to a valid field
            switch (minifield(fields{fi}))
            case 'tri'
                k.st(ck).trials = v;
            case 'con'
                k.st(ck).conds = v;
            case 'blo'
                k.st(ck).blocks = v;
            case 'res'
                k.st(ck).resps = v;
            case 'typ'
                k.st(ck).type  = v;
            case 'exp'
                k.st(ck).expect = v;
            case 'rep'
                k.st(ck).rept = v;
            case 'cue'
                k.st(ck).cues = v;
            case 'sta'
                if length (v) > 1, error ('START field must be a scalar'); end
                k.st(ck).start = v;
            case 'fin'
                if length (v) > 1, error ('FINISH field must be a scalar'); end
                k.st(ck).finish = v;
            otherwise
                error ('bad field name in header');
            end
        end % resume here after having processed all fields in oneline
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
otherwise
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    error ('wrong number of arguments');
end

%finish up
k = class (k, 'key');
