function sp = addlevel (sp, factor, level, type, value)

%@SPIX/ADDLEVEL   defines a factor level
%
%   Syntax 1: SP = ADDLEVEL(SP,FACTOR,LEVEL,KEYS)
%   A level is a key or a set of keys corresponding to experimental
%   conditions that share a common feature (e.g., all red targets). A
%   factor is a set of two or more levels whose keys do not intersect
%   (e.g., a 'color' factor containing 'red', 'blue', and 'green' levels).
%   The string FACTOR is the factor's label. If FACTOR already exists in
%   SP, then the level is added to it, otherwise a new factor is created.
%   The string LEVEL is the level's label. If LEVEL already exists in
%   FACTOR, then the level is redefined, otherwise a new level is created.
%   KEYS can be a key label, a set of key labels (in a cell array of
%   strings), one of 'none' 'all' 'first' 'last', or a numeric array
%   specifying the ordinal numbers of the keys to be selected.
%
%   Syntax 2: SP = ADDLEVEL(SP,FACTOR,LEVEL,TYPE,VALUE)
%   This syntax allows using a previously created combo or group of keys to
%   define a level; valid TYPEs are 'group', 'combo', and 'keys' (which is
%   equivalent to using Syntax 1). VALUE is the label or the ordinal number
%   of the group/combo.
%
%Last modified: 12 Sep 03

switch nargin
   case 4
      keys = parsegrouping (sp.k, type);
   case 5
      switch minifield (type)
         case 'key'
            keys = parsegrouping (sp.k, value);
         case 'gro'
            group = parsegrouping (sp.groups, value);
            if ~isnumscalar (group), error ('only one group, please'); end
            keys = sp.groups(group).keys;
         case 'com'
            combo = parsegrouping (sp.combos, value);
            if ~isnumscalar (combo), error ('only one combo, please'); end
            keys = sp.groups(combo).keys;
         otherwise
            error ('invalid TYPE');
      end
   otherwise
      error ('wrong number of arguments');
end

if ~isstring (factor), error ('FACTOR must be a char label'); end
if ~isstring (level), error ('LEVEL must be a char label'); end

fi = lbl2idx (sp.factors, factor);
if isempty (fi)
   disp (['creating a new factor labeled ''' factor '''']);
   fi = sp.nfactors + 1;
   sp.factors(fi).label = factor;
   sp.factors(fi).levels = [];
   sp.factable (fi,sp.k.len)=0;
   sp.nfactors = fi;
else
   disp (['using the existing factor labeled ''' factor '''']);
end

le = lbl2idx (sp.factors(fi).levels, level);
if isempty (le)
   disp (['creating a new level labeled ''' level ''' with the following keys:']);
   le = length (sp.factors(fi).levels) + 1;
else
   disp (['redefining level labeled ''' level ''' with the following keys:']);
end

sp.factors(fi).levels(le).label = level;
sp.factors(fi).levels(le).keys  = keys;
sp.factable(fi,keys) = le;
disp (keys);