function [dense, uuni, ugrp] = density (sp, win, bin, kwidth, ktype, varargin)

%SPIX/DENSITY  spike-density functions from a @spix object
%
%   [DE,UNI,GRP] = DENSITY(SP,WIN,BIN,WIDTH,TYPE)
%   SP is a @spix object; WIN is a 2-element vector specifying the start
%   and end ms for the analysis, relative to the cue event; WIDTH is the
%   size in ms of the kernel (larger values produce smoother curves); TYPE
%   is the shape of the kernel and is either 'gauss' or 'boxcar'.
%
%   DE is a 3-D matrix of density values; each row represents one processed
%   unit; each column corresponds to one ms; density plots for different
%   keys or key groupings are arranged along the 3rd dimension.
%
%   If no other arguments are specified, then all currently included units
%   are processed for each of the currently included keys.
%
%   Pairs of 'specs' can be appended to the argument list in order to
%   restrict analysis to certain units and-or keys.
%
%   For example, density(sp,[-100,299],20,'gauss','uni',6:10,'key',1:3)
%   will return a matrix of size (5, 400, 3)
%
%   [DE,UNI,GRP] = DENSITY(...)
%   UNI is a list of units included in the analysis
%   GRP is a list of keys or key groupings analyzed
%
%Last modified: 23 Aug 03

[spmat, uni, grp] = spikematrix (sp, win, bin, varargin{:});
uuni = unique(uni); nuni = length(uuni);
ugrp = unique(grp); ngrp = length(ugrp);
dense = zeros (nuni, size(spmat,2), ngrp);
for cc = 1:length(uuni)
   for gr = 1:length(ugrp)
      chunk = spmat (uni==uuni(cc) & grp==ugrp(gr), :);
      te = smoothfr (chunk, kwidth, ktype);
      dense(cc,:,gr) = te;
   end
end
