function s = spix (k,n)

%@SPIX/SPIX   the @spix object class lets you combine @cortex objects
%   into a single entity, allowing efficient spike data analysis on
%   any number of units at once. Currently, the @spix object does NOT
%   handle behavioral data.
%
%Constructor's syntax:
%   SPOB = SPIX(KEYS)   creates a @spix object and assigns it to SPOB.
%   KEYS is either a @key object containing any number of keys, or a
%   string containing the path/filename of a .key source text file.
%   All keys must specify time windows of identical duration. All
%   analysis epochs in the @spix will be prepared and later on sorted
%   on the basis of  these keys. Thus, do not combine different
%   experimental conditions in a single key if you think that later on
%   you might want to analyze those conditions separately. Also, use
%   as large an epoch as possible.
%
%   SPOB = SPIX(KEYS,NAME)   the string NAME is used as object label;
%   if NAME is not specified, the default 'noname' string is used.
%
%Last modified: 9 Aug 03

% 18 apr 02: last unnoted changes
% 24 dec 02: switch nargin according to matlab standards
% 24 Apr 03: resuming work on filters
% 26 Apr 03: groups of keys introduced
%  9 Aug 03: combos introduced; version reverted to 1, assuming that nobody
%            has saved any important objects yet.

error (nargchk(0,2,nargin));

s.name = 'noname';
if nargin == 0
   k = key;
else
   if isa (k, 'spix')
      s = k;
      return
   end
   if nargin==2
      if ~ischar (n), error ('wrong input argument'), end
      s.name = n;
   end
end

s.version = 2;
[s.k s.win] = getkeys (k);

s.verbose = true;
s.nunits = 0;
s.nsubs = 0;
s.ngroups = 0;
s.ncombos = 0;
s.nfactors = 0;
% needed to place the 0 on the time scale
s.offset = s.k(1,1).sta;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% a single, large, sparse matrix for the whole obj
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
s.spix   = sparse (0, s.win);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 'fixed' arrays whose values,
% set when adding units, never change
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% these are as long as the total number of trials in spix
s.unit = uint16([]);  % unit number
s.key  = uint16([]);  % key number
s.trial = uint16([]); % chronological trial number
s.yest = logical([]); % included trials

s.info = [];
% % s.info(1).label  = [];
% % s.info(1).source = [];
% % s.info(1).date   = [];
% % s.info(1).extra  = [];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 'dynamic' arrays, used to include/exclude data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
s.inclunits = [];
s.inkeys    = 1:s.k.len;

% subsets of units
s.subsets = [];
% % s.subsets(1).label = '';
% % s.subsets(1).units = [];

% groups of keys
s.groups = [];
% % s.groups(1).label = '';
% % s.groups(1).keys  = [];

% combinations of keys
s.combos = [];
% % s.combos(1).label = '';
% % s.combos(1).keys  = [];

% grouping variables, e.g. for anovas
s.factors = [];
% % s.factors(1).label = '';
% % s.factors(1).levels(1).label = '';
% % s.factors(1).levels(1).keys = [];

% subsets of trials that may not match key grouping
s.filters = [];
% % s.filters(1).label = '';
% % s.filters(1).notes = '';
% % s.filters(1).array = uint8([]);

% factor and level synopsis
s.factable = [];

% extra information
s.latency = [];

%%%%%%%%%%%%%%%
% battezza spix
%%%%%%%%%%%%%%%
s = class(s, 'spix');
