function [frates, instances] = meanfiringrate (spmat, binsize)

%MEANFIRINGRATE  average firing rates from a spikematrix.
%
%   [FRATES,INSTANCES] = MEANFIRINGRATE(SPMAT,BINSIZE) computes average
%      firing rates (spikes/second) on the SPMAT spike matrix (see help
%      spikematrix for more information on spikematrices). If BINSIZE is
%      omitted, then a single mean is computed from the whole spikematrix;
%      otherwise, the width of the spikematrix is divided in bins of
%      BINSIZE width (in ms), and separate firing rates are computed on
%      each bin. Any remainder milliseconds (trailing spikematrix columns)
%      after the division are ignored. FRATES is a row vector of average
%      firing rates, one per bin. INSTANCES is the number of spike arrays
%      in the spikematrix (equivalent to SIZE(SPMAT,1))
%
% Last modified: 19 Sep 03

% 29 mar 00: added single argument call (spm)
% 14 Jul 00: help text fixes
%  9 Nov 02: overloaded function; see also @cortex\meanfiringrate.m
% 16 Apr 03: help text
% 17 Apr 03: call to truncspm
% 22 Apr 03: sum(spmat, 1) bugfix
% 19 Sep 03: help text revised

switch nargin
   case 1
      binsize = size (spmat, 2);
   case 2
      if prod (size(binsize))>1
         error ('binsize should be a scalar');
      end
   otherwise
      error ('wrong number of arguments');
end

% make sure that the length of the array is a multiple of binsize
[spmat, instances, winsize, numofbins] = truncspm (spmat, binsize);

% compute the firing rates
spmat = reshape(sum(spmat, 1), binsize, numofbins);
spptr = double(sum(spmat, 1))/instances;
frates  = spptr / binsize * 1000;